"use strict";(self.webpackChunkappyter_js=self.webpackChunkappyter_js||[]).push([[643],{1643:(t,e,n)=>{n.r(e),n.d(e,{default:()=>f});var l=n(7418);function o(t){(0,l.kZQ)(t,"svelte-12243wq",".toc.svelte-12243wq{display:block;font-weight:bold}.toc.h1.svelte-12243wq{font-size:100%;text-indent:0em}.toc.h2.svelte-12243wq{font-size:90%;text-indent:1em}.toc.h3.svelte-12243wq{font-size:80%;text-indent:1.5em}.toc.h4.svelte-12243wq{font-size:70%;text-indent:1.75em}.toc.h5.svelte-12243wq{font-size:60%;text-indent:1.85em}.toc.h6.svelte-12243wq{font-size:50%;text-indent:2em}.sticky-top.svelte-12243wq{z-index:1020}")}function s(t,e,n){const l=t.slice();return l[1]=e[n].h,l[2]=e[n].id,l[3]=e[n].textContent,l}function c(t){let e,n,o,c,r,d,u=(0,l.rv_)(t[0]),f=[];for(let e=0;e<u.length;e+=1)f[e]=i(s(t,u,e));return{c(){e=(0,l.ND4)("div"),n=(0,l.ND4)("div"),o=(0,l.ND4)("div"),c=(0,l.ND4)("div"),r=(0,l.ND4)("legend"),r.textContent="Table Of Contents",d=(0,l.xem)();for(let t=0;t<f.length;t+=1)f[t].c();(0,l.CFu)(c,"class","mt-5"),(0,l.CFu)(o,"class","offset-sm-2 col-sm-8 col-md-12"),(0,l.CFu)(n,"class","row sticky-top svelte-12243wq"),(0,l.CFu)(e,"class","col-sm-12 col-md-3 col-xl-2")},m(t,s){(0,l.Yry)(t,e,s),(0,l.BCw)(e,n),(0,l.BCw)(n,o),(0,l.BCw)(o,c),(0,l.BCw)(c,r),(0,l.BCw)(c,d);for(let t=0;t<f.length;t+=1)f[t]&&f[t].m(c,null)},p(t,e){if(1&e){let n;for(u=(0,l.rv_)(t[0]),n=0;n<u.length;n+=1){const l=s(t,u,n);f[n]?f[n].p(l,e):(f[n]=i(l),f[n].c(),f[n].m(c,null))}for(;n<f.length;n+=1)f[n].d(1);f.length=u.length}},d(t){t&&(0,l.YoD)(e),(0,l.ppq)(f,t)}}}function i(t){let e,n,o,s,c,i=t[3]+"";return{c(){e=(0,l.ND4)("a"),n=(0,l.Qq7)(i),o=(0,l.xem)(),(0,l.CFu)(e,"href",s="#"+t[2]),(0,l.CFu)(e,"class",c="toc h"+t[1]+" svelte-12243wq")},m(t,s){(0,l.Yry)(t,e,s),(0,l.BCw)(e,n),(0,l.BCw)(e,o)},p(t,o){1&o&&i!==(i=t[3]+"")&&(0,l.iQh)(n,i),1&o&&s!==(s="#"+t[2])&&(0,l.CFu)(e,"href",s),1&o&&c!==(c="toc h"+t[1]+" svelte-12243wq")&&(0,l.CFu)(e,"class",c)},d(t){t&&(0,l.YoD)(e)}}}function r(t){let e,n=void 0!==t[0]&&c(t);return{c(){n&&n.c(),e=(0,l.Iex)()},m(t,o){n&&n.m(t,o),(0,l.Yry)(t,e,o)},p(t,[l]){void 0!==t[0]?n?n.p(t,l):(n=c(t),n.c(),n.m(e.parentNode,e)):n&&(n.d(1),n=null)},i:l.lQ1,o:l.lQ1,d(t){t&&(0,l.YoD)(e),n&&n.d(t)}}}function d(t,e,n){let{toc:l}=e;return t.$$set=t=>{"toc"in t&&n(0,l=t.toc)},[l]}n(5866);class u extends l.r7T{constructor(t){super(),(0,l.TsN)(this,t,d,r,l.jXN,{toc:0},o)}}const f=u}}]);