"use strict";(self.webpackChunkappyter_js=self.webpackChunkappyter_js||[]).push([[20],{6020:(e,o,n)=>{async function l(e){const{default:o}=await Promise.all([n.e(16),n.e(130)]).then(n.bind(n,6130));let l=!0;o.on("connect",()=>{l?(console.debug("connected"),l=!1):(console.debug("reconnect"),e.location.reload())}),o.on("livereload",o=>{console.debug("livereload"),e.location.reload()})}n.r(o),n.d(o,{default:()=>l})}}]);