{% extends "default/form.j2" %}

{% block body %}
<div id="_content" class="svelte container loading">
  {% include "fragments/loader.j2" %}
  <script type="application/json">{{ dict(fields=_fields, js_url=url_for('static', filename='js/form.js'))|jsonify }}</script>
</div>
{% endblock %}
